@@ -5,7 +5,7 @@
  */
 
 #ifdef FORBMP
-#include <beep-media-player/bmp/configfile.h>
+#include <bmp/configfile.h>
 #else
 #include <xmms/configfile.h>
 #endif
@@ -16,6 +16,7 @@
 #include "bio2jack.h" /* includes for the bio2jack library */
 #include "jack.h"
 #include "xconvert.h" /* xmms rate conversion header file */
+#include <string.h>
 
 
 
@@ -176,6 +177,25 @@ gint jack_playing(void)
 }
 
 
+void jack_set_port_connection_mode()
+{
+  /* setup the port connection mode that determines how bio2jack will connect ports */
+  enum JACK_PORT_CONNECTION_MODE mode;
+  
+  if(strcmp(jack_cfg.port_connection_mode, "CONNECT_ALL") == 0)
+      mode = CONNECT_ALL;
+  else if(strcmp(jack_cfg.port_connection_mode, "CONNECT_OUTPUT") == 0)
+      mode = CONNECT_OUTPUT;
+  else if(strcmp(jack_cfg.port_connection_mode, "CONNECT_NONE") == 0)
+      mode = CONNECT_NONE;
+  else
+  {
+      TRACE("Defaulting to CONNECT_ALL");
+      mode = CONNECT_ALL;
+  }
+  JACK_SetPortConnectionMode(mode);
+}
+
 /* Initialize necessary things */
 void jack_init(void)
 {
@@ -190,9 +210,15 @@ void jack_init(void)
   #endif
   cfgfile = xmms_cfg_open_file(filename);
   if (!cfgfile)
+  {
       jack_cfg.isTraceEnabled = FALSE;
-  else
+      jack_cfg.port_connection_mode = "CONNECT_ALL"; /* default to connect all */
+  } else
+  {
       xmms_cfg_read_boolean(cfgfile, "jack", "isTraceEnabled", &jack_cfg.isTraceEnabled);
+      if(!xmms_cfg_read_string(cfgfile, "jack", "port_connection_mode", &jack_cfg.port_connection_mode))
+          jack_cfg.port_connection_mode = "CONNECT_ALL";
+  }
 
   xmms_cfg_free(cfgfile);
   g_free(filename);
@@ -205,6 +231,9 @@ void jack_init(void)
   /* jack client list */
   JACK_SetClientName("xmms-jack");
 
+  /* set the port connection mode */
+  jack_set_port_connection_mode();
+
   xmmslibhandle = dlopen("libxmms.so", RTLD_NOW);
   if(xmmslibhandle)
   {
@@ -261,8 +290,8 @@ void jack_init(void)
 /* Return the amount of data that can be written to the device */
 gint jack_free(void)
 {
-  long return_val = JACK_GetBytesFreeSpace(driver);
-  long tmp;
+  unsigned long return_val = JACK_GetBytesFreeSpace(driver);
+  unsigned long tmp;
 
   /* adjust for frequency differences, otherwise xmms could send us */
   /* as much data as we have free, then we go to convert this to */
@@ -275,7 +304,14 @@ gint jack_free(void)
     TRACE("adjusting from %ld to %ld free bytes to compensate for frequency differences\n", tmp, return_val);
   }
 
+  if(return_val > G_MAXINT)
+  {
+      TRACE("Warning: return_val > G_MAXINT\n");
+      return_val = G_MAXINT;
+  }
+
   TRACE("free space of %ld bytes\n", return_val);
+
   return return_val;
 }
 
@@ -307,6 +343,7 @@ gint jack_open(AFormat fmt, gint sample_rate, gint num_channels)
 {
   int bits_per_sample;
   int retval;
+  unsigned long rate;
 
   TRACE("fmt == %d, sample_rate == %d, num_channels == %d\n",
 	fmt, sample_rate, num_channels);
@@ -335,16 +372,20 @@ gint jack_open(AFormat fmt, gint sample_rate, gint num_channels)
   /* if we are already opened then don't open again */
   if(output_opened)
   {
-    /* if the number of channels has changed, notify bio2jack */
-    if(output.channels != input.channels)
+    /* if something has changed we should close and re-open the connect to jack */
+    if((output.channels != input.channels) ||
+       (output.frequency != input.frequency) ||
+       (output.format != input.format))
     {
-      TRACE("output.channels is %d, jack_open called with %d channels\n", output.channels, num_channels);
-      output.channels = input.channels;
-      JACK_SetNumInputChannels(driver, output.channels); /* match bio2jack's input channels with xmms's output channels */
+      TRACE("output.channels is %d, jack_open called with %d channels\n", output.channels, input.channels);
+      TRACE("output.frequency is %ld, jack_open called with %ld\n", output.frequency, input.frequency);
+      TRACE("output.format is %d, jack_open called with %d\n", output.format, input.format);
+      jack_close();
+    } else
+    {
+        TRACE("output_opened is TRUE and no options changed, not reopening\n");
+        return 1;
     }
-
-    TRACE("output_opened is TRUE, not reopening\n");
-    return 1;
   }
 #endif /* #if HAVE_CTOR */
 
@@ -352,13 +393,21 @@ gint jack_open(AFormat fmt, gint sample_rate, gint num_channels)
   output.frequency = input.frequency;
   output.bps       = input.bps;
   output.channels  = input.channels;
-  retval = JACK_Open(&driver, bits_per_sample, &output.frequency, output.channels);
+  output.format    = input.format;
+
+  rate = output.frequency;
+  retval = JACK_Open(&driver, bits_per_sample, &rate, output.channels);
+  output.frequency = rate; /* avoid compile warning as output.frequency differs in type
+                              from what JACK_Open() wants for the type of the rate parameter */
   if((retval == ERR_RATE_MISMATCH) && isXmmsFrequencyAvailable)
   {
     TRACE("xmms(input) wants rate of '%ld', jacks rate(output) is '%ld', opening at jack rate\n", input.frequency, output.frequency);
 
     /* open the jack device with true jack's rate, return 0 upon failure */
-    if((retval = JACK_Open(&driver, bits_per_sample, &output.frequency, output.channels)))
+    retval = JACK_Open(&driver, bits_per_sample, &rate, output.channels);
+    output.frequency = rate; /* avoid compile warning as output.frequency differs in type
+                                from what JACK_Open() wants for the type of the rate parameter */
+    if(retval)
     {
       TRACE("failed to open jack with JACK_Open(), error %d\n", retval);
       return 0;
@@ -457,7 +506,7 @@ void jack_write(gpointer ptr, gint length)
   while(length > 0)
   {
     TRACE("writing %d bytes\n", length);
-    written = JACK_Write(driver, buf, length);
+    written = JACK_Write(driver, (unsigned char*)buf, length);
     length-=written;
     buf+=written;
   }
@@ -500,20 +549,42 @@ void jack_pause(short p)
 /* Set the volume */
 void jack_set_volume(int l, int r)
 {
-  TRACE("l(%d), r(%d)\n", l, r);
-  JACK_SetVolumeForChannel(driver, 0, l);
-  JACK_SetVolumeForChannel(driver, 1, r);
+  if(output.channels == 1)
+  {
+    TRACE("l(%d)\n", l);
+  } else if(output.channels > 1)
+  {
+    TRACE("l(%d), r(%d)\n", l, r);
+  }
+
+  if(output.channels > 0)
+      JACK_SetVolumeForChannel(driver, 0, l);
+  if(output.channels > 1)
+      JACK_SetVolumeForChannel(driver, 1, r);
 }
 
 
 /* Get the current volume setting */
 void jack_get_volume(int *l, int *r)
 {
-  JACK_GetVolumeForChannel(driver, 0, l);
-  JACK_GetVolumeForChannel(driver, 1, r);
+  unsigned int _l, _r;
+
+  if(output.channels > 0)
+  {
+      JACK_GetVolumeForChannel(driver, 0, &_l);
+      (*l) = _l;
+  }
+  if(output.channels > 1)
+  {
+      JACK_GetVolumeForChannel(driver, 1, &_r);
+      (*r) = _r;
+  }
 
 #if VERBOSE_OUTPUT
-  TRACE("l(%d), r(%d)\n", *l, *r);
+  if(output.channels == 1)
+      TRACE("l(%d)\n", *l);
+  else if(output.channels > 1)
+      TRACE("l(%d), r(%d)\n", *l, *r);
 #endif
 }
 
@@ -521,10 +592,10 @@ void jack_get_volume(int *l, int *r)
 void jack_about(void)
 {
 	dialog = gtk_dialog_new();
-	gtk_window_set_title(GTK_WINDOW(dialog), ("About jack Driver 0.14"));
+	gtk_window_set_title(GTK_WINDOW(dialog), ("About jack Driver 0.15"));
 	gtk_container_border_width(GTK_CONTAINER(dialog), 5);
 	label = gtk_label_new((
-		"XMMS jack Driver 0.14\n\n "
+		"XMMS jack Driver 0.15\n\n "
 		"xmms-jack.sf.net\n"
 		"Chris Morgan<cmorgan@alum.wpi.edu>\n"));
 
