@@ -7,7 +7,7 @@
 
 
 #ifdef FORBMP
-#include <beep-media-player/bmp/plugin.h>
+#include <bmp/plugin.h>
 #else
 #include <xmms/plugin.h>
 #endif
